﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Threading;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Text;
using System.Windows.Forms;

namespace SortMethods
{
    public partial class Form1 : Form
    {
        const int N = 40; // Число элементов
        const int M = 50; // Диапазон чисел

        int[] A = new int[N];

        bool running = false;
        Thread T;

        int DemoTick = 100;
        int SwapTick = 150;

        int State = 0;
        int I, J;
        int P, Q;

        public Form1()
        {
            InitializeComponent();
            SetStyle(ControlStyles.ResizeRedraw, true);
            newToolStripMenuItem.PerformClick();
        }

        protected override void OnPaint(PaintEventArgs e)
        {
            panel1.Invalidate();
        }

        protected override void OnClosing(CancelEventArgs e)
        {
            if (running) T.Abort();
            base.OnClosing(e);
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void newToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Random R = new Random();
            for (int i = 0; i < N; i++) A[i] = R.Next(M) + 1;
            panel1.Invalidate();
        }

        private void PaintPanel(object sender, PaintEventArgs e)
        {
            Graphics G = e.Graphics;
            int H = panel1.Height; float W = panel1.Width;
            float w = (W-1)/N;
            float h = ((float) H -1) / M;
            G.TranslateTransform(0, H-1);
            G.ScaleTransform(1, -1);

            Brush bn = new LinearGradientBrush
                (new Point(0, 0), new Point(0, H - 1), 
                 Color.Blue, Color.Red);
            Brush bc = new LinearGradientBrush
                (new Point(0, 0), new Point(0, H - 1),
                 Color.Green, Color.Blue);
            Brush bs = new LinearGradientBrush
                (new Point(0, 0), new Point(0, H - 1),
                 Color.Red, Color.Yellow);

            for (int i = 0; i < N; i++)
            {
                Brush b;

                if (State != 0 && ((i == I) || (i == J)))
                {
                    if (State == 1) b = bs; else b = bc;
                }
                else b = bn;

                G.FillRectangle(b, i * w, 0, w, A[i] * h);
                G.DrawRectangle(Pens.LightGray, i * w, 0, w, A[i] * h);
            }
        }

        private void linearToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ToolStripMenuItem m = (ToolStripMenuItem)sender;
            switch (m.Name)
            {
                case "linearToolStripMenuItem":
                    T = new Thread(new ThreadStart(Linear));
                    break;
                case "bubbleToolStripMenuItem":
                    T = new Thread(new ThreadStart(Bubble));
                    break;
                case "shakerToolStripMenuItem":
                    T = new Thread(new ThreadStart(Shaker));
                    break;
                case "quickToolStripMenuItem":
                    T = new Thread(new ThreadStart(Quick));
                    break;
            }
            T.Start();

        }

        private void swap(int i, int j)
        {
            I = i; J = j; State = 1; Q++;
            panel1.Invalidate(); Thread.Sleep(SwapTick);
            int X = A[i]; A[i] = A[j]; A[j] = X;
            panel1.Invalidate(); Thread.Sleep(SwapTick);
            State = 0;
        }

        private void demo(int i, int j)
        {
            I = i; J = j; State = 2; P++;
            panel1.Invalidate(); Thread.Sleep(DemoTick);
            State = 0; 
        }

        private void Linear()
        {
            P = 0; Q = 0;
            if (running) return;
            running = true;

            for (int i=0; i<N-1; i++)
                for (int j = i + 1; j < N; j++)
                {
                    demo(i, j);
                    if (A[i] > A[j]) swap(i, j);
                }
            panel1.Invalidate();
            MessageBox.Show("Просмотров " + P.ToString() +
                ", перестановок " + Q.ToString(), 
                "Линейная сортировка",
                MessageBoxButtons.OK,
                MessageBoxIcon.Asterisk);
            running = false;
        }

        private void Bubble()
        {
            P = 0; Q = 0;
            if (running) return;
            running = true;
            bool f;

            for (int i = N - 1; i > 0; i--)
            {   f = true;
                for (int j = 0; j < i; j++)
                {
                    demo(j, j + 1);
                    if (A[j] > A[j + 1])
                    {
                        swap(j, j + 1); f = false;
                    }
                }
                if (f) break;
            }
            panel1.Invalidate();
            MessageBox.Show("Просмотров " + P.ToString() +
                ", перестановок " + Q.ToString(),
                "Сортировка пузрьком",
                MessageBoxButtons.OK,
                MessageBoxIcon.Asterisk);
            running = false;
        }

        private void Shaker()
        {
            P = 0; Q = 0;
            if (running) return;
            running = true;
            bool f;

            int L = 0; int R = N-1;

            while (true) 
            {
                f = true; // ==>>>>>
                for (int j = L; j < R; j++)
                {
                    demo(j, j + 1);
                    if (A[j] > A[j + 1])
                    {  swap(j, j + 1); f = false;
                    }
                }
                if (f) break;
                R--;

                f = true; // <<<<<<<====
                for (int j = R-1; j >= L; j--)
                {
                    demo(j, j + 1);
                    if (A[j] > A[j + 1])
                    {
                        swap(j, j + 1); f = false;
                    }
                }
                if (f) break;
                L++;


            }
            panel1.Invalidate();
            MessageBox.Show("Просмотров " + P.ToString() +
                ", перестановок " + Q.ToString(),
                "Шейкер сортировка",
                MessageBoxButtons.OK,
                MessageBoxIcon.Asterisk);
            running = false;
        }

        private void Quick()
        {
            if (running) return;
            running = true;
            P = 0; Q = 0;
            QuickSort(0, N - 1);
            panel1.Invalidate();
            MessageBox.Show("Просмотров " + P.ToString() +
                ", перестановок " + Q.ToString(),
                "Быстрая сортировка",
                MessageBoxButtons.OK,
                MessageBoxIcon.Asterisk);
            running = false;
        }

        private void QuickSort(int Left, int Right)
        {
            int L = Left; int R = Right;
            int nc = (L + R) / 2;
            int C = A[nc];

            while (L<=R)
            {
             while (A[L] < C)  { demo(L, nc); L++; }
             while (A[R] > C) { demo(R, nc); R--; }
             if (L <= R) { swap(L, R); L++; R--; }
            }
            if (Left < R) QuickSort(Left, R);
            if (L < Right) QuickSort(L, Right);
        }


        private void pauseToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (running)
            {
                if (!pauseToolStripMenuItem.Checked)
                {
                    T.Suspend();
                    pauseToolStripMenuItem.Checked = true;
                }
                else
                {
                    T.Resume();
                    pauseToolStripMenuItem.Checked = false;
                }
            }
        }
    }
}
